<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'filter_tex', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package    filter
 * @subpackage tex
 * @copyright  1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['configconvertformat'] = 'If <i>latex</i> and <i>dvips</i> are present in addition to <i>convert</i> or <i>dvisvgm</i>, select the preferred image type (<i>convert</i> produces GIF or PNG; <i>dvisvgm</i> produces SVG). Otherwise <i>mimeTeX</i> will be used to to create GIF images.';
$string['convertformat'] = 'Output image format';
$string['latexpreamble'] = 'LaTeX preamble';
$string['latexsettings'] = 'LaTeX renderer Settings';
$string['filtername'] = 'TeX notation';
$string['pathconvert'] = 'Path of <i>convert</i> binary';
$string['pathdvips'] = 'Path of <i>dvips</i> binary';
$string['pathdvisvgm'] = 'Path of <i>dvisvgm</i> binary';
$string['pathlatex'] = 'Path of <i>latex</i> binary';
$string['pathmimetex'] = 'Path of <i>mimetex</i> binary';
$string['pathmimetexdesc'] = 'Moodle will use its own mimetex binary unless another valid path is specified.';
$string['source'] = 'TeX source';
$string['privacy:metadata'] = 'The TeX notation plugin does not store any personal data.';
